"""Reverso.net API for Python"""

from . import context, voice


__author__ = "Demian Wolf"
__copyright__ = "Copyright 2020, Demian Wolf"
__credits__ = ["Demian Wolf"]
__license__ = "MIT"
__version__ = "0.0.1.beta.3"
__maintainer__ = "Demian Wolf"
__email__ = "demianwolfssd@gmail.com"
